@@ -1,10 +1,6 @@
-/** 
-
-TODO: 
-  - if ignorePrefix, should highlight current value (not the 1st one)
-      a) change in _updateChoicesFunction (because there might be an
-         initial value in the form)
+/**
 
+TODO:
   - messages : choose language
 
   - multivalue :
@@ -50,11 +46,11 @@ GvaScript.AutoCompleter = function(datasource, options) {
     multivalue_separator :  /[;,\s]\s*/,
     choiceItemTagName: "div",
     htmlWrapper      : function(html) {return html;},
-    observed_scroll  : null,      // observe the scroll of a given element and 
-                                  // move the dropdown accordingly (useful in 
+    observed_scroll  : null,      // observe the scroll of a given element and
+                                  // move the dropdown accordingly (useful in
                                   // case of scrolling windows)
     additional_params: null,      // additional parameters with optional default
-                                  // values (only in the case where the 
+                                  // values (only in the case where the
                                   // datasource is a URL)
     http_method      : 'get',     // method for Ajax requests
     dependentFields  : {},
@@ -63,16 +59,17 @@ GvaScript.AutoCompleter = function(datasource, options) {
   };
 
   // more options for array datasources
-  if (typeof datasource == "object" && datasource instanceof Array) { 
-    defaultOptions.ignorePrefix  = false;  // if true, will always display 
+  if (typeof datasource == "object" && datasource instanceof Array) {
+    defaultOptions.ignorePrefix  = false;  // if true, will always display
                                            // the full list
     defaultOptions.caseSensitive = true;
   }
 
   this.options = Class.checkOptions(defaultOptions, options);
 
-  // backwards compatibility
-  this.options.checkNewValDelay = this.options.autoSuggestDelay;
+  // autoSuggestDelay cannot be smaller than checkNewValueDelay
+  this.options.autoSuggestDelay = Math.max(this.options.autoSuggestDelay,
+                                           this.options.checkNewValDelay);
 
   var defaultClasses = {
     loading         : "AC_loading",
@@ -80,7 +77,7 @@ GvaScript.AutoCompleter = function(datasource, options) {
     message         : "AC_message"
   };
   this.classes = Class.checkOptions(defaultClasses, this.options.classes);
-  
+
   if (this.options.multivalued && this.options.strict) {
     throw new Error("options 'strict' and 'multivalue' are incompatible");
   }
@@ -116,19 +113,22 @@ GvaScript.AutoCompleter.prototype = {
 // PUBLIC METHODS
 //----------------------------------------------------------------------
 
-  // autocomplete : called when the input element gets focus; binds 
-  // the autocompleter to the input element 
-  autocomplete: function(elem) { 
+  // autocomplete : called when the input element gets focus; binds
+  // the autocompleter to the input element
+  autocomplete: function(elem) {
     elem = $(elem);// in case we got an id instead of an element
 
     if (!elem) throw new Error("attempt to autocomplete a null element");
 
+    // elem is readonly => no action
+    if (elem.getAttribute('readonly') || elem.readOnly) return;
+
     // if already bound, no more work to do
     if (elem === this.inputElement) return;
 
     // bind to the element; if first time, also register the event handlers
     this.inputElement = elem;
-    if (!elem._autocompleter) { 
+    if (!elem._autocompleter) {
       elem._autocompleter = this;
       this.keymap.observe("keydown", elem, Event.stopNone);
       Element.observe(elem, "blur", this.reuse.onblur);
@@ -141,7 +141,7 @@ GvaScript.AutoCompleter.prototype = {
     // initialize time stamps
     this._timeLastCheck = this._timeLastKeyDown = 0;
 
-    // more initialization, but only if we did not just come back from a 
+    // more initialization, but only if we did not just come back from a
     // click on the dropdownDiv
     if (!this.dropdownDiv) {
       this.lastTypedValue = this.lastValue = "";
@@ -170,15 +170,15 @@ GvaScript.AutoCompleter.prototype = {
   // set additional params for autocompleters that have more than 1 param;
   // second param is the HTTP method (post or get)
   // DALNOTE 10.01.09 : pas de raison de faire le choix de la  mthode HTTP
-  // dans  setAdditionalParams()! TOFIX. Apparemment, utilis une seule fois 
+  // dans  setAdditionalParams()! TOFIX. Apparemment, utilis une seule fois
   // dans DMWeb (root\src\tab_composition\form.tt2:43)
   setAdditionalParams : function(params, method) {
-    this.additional_params = params;           
+    this.additional_params = params;
     if (method) this.options.http_method = method;
   },
 
   addAdditionalParam : function(param, value) {
-    if (!this.additional_params) 
+    if (!this.additional_params)
       this.additional_params = {};
     this.additional_params[param] = value;
   },
@@ -190,31 +190,31 @@ GvaScript.AutoCompleter.prototype = {
 
     // register proper "updateChoices" function according to type of datasource
     var ds_type = typeof datasource;
-    this._updateChoicesHandler 
+    this._updateChoicesHandler
       = (ds_type == "string")   ? this._updateChoicesFromAjax
       : (ds_type == "function") ? this._updateChoicesFromCallback
-      : (ds_type == "object" && datasource instanceof Array) 
-                                ? this._updateChoicesFromArray 
-      : (ds_type == "object" && datasource instanceof Object) 
+      : (ds_type == "object" && datasource instanceof Array)
+                                ? this._updateChoicesFromArray
+      : (ds_type == "object" && datasource instanceof Object)
                                 ? this._updateChoicesFromJSONP
       : undefined;
      if (!this._updateChoicesHandler)
       throw new Error("unexpected datasource type");
   },
 
-  // 'fireEvent' function is copied from GvaScript.fireEvent, so that "this" 
+  // 'fireEvent' function is copied from GvaScript.fireEvent, so that "this"
   // in that code gets properly bound to the current object
-  fireEvent: GvaScript.fireEvent, 
+  fireEvent: GvaScript.fireEvent,
 
   // Set the element for the AC to look at to adapt its position. If elem is
   // null, stop observing the scroll.
-  // DALNOTE 10.01.09 : pas certain de l'utilit de "set_observed_scroll"; si 
+  // DALNOTE 10.01.09 : pas certain de l'utilit de "set_observed_scroll"; si
   // l'lment est positionn correctement dans le DOM par rapport  son parent,
   // il devrait suivre le scroll automatiquement. N'est utilis dans DMWeb que
   // par "avocat.js".
   set_observed_scroll : function(elem) {
     if (!elem) {
-        Event.stopObserving(this.observed_scroll, 'scroll', 
+        Event.stopObserving(this.observed_scroll, 'scroll',
                             correct_dropdown_position);
         return;
     }
@@ -229,13 +229,13 @@ GvaScript.AutoCompleter.prototype = {
         pos.top  -= this.observed_scroll.scrollTop - this.currentScrollTop;
         pos.left -= this.observed_scroll.scrollLeft;
         this.dropdownDiv.style.top  = pos.top   + "px";
-        this.dropdownDiv.style.left = pos.left  + "px"; 
+        this.dropdownDiv.style.left = pos.left  + "px";
       }
       this.currentScrollTop = this.observed_scroll.scrollTop;
       this.currentScrollLeft = this.observed_scroll.scrollLeft;
     }
 
-    Event.observe(elem, 'scroll', 
+    Event.observe(elem, 'scroll',
                   correct_dropdown_position.bindAsEventListener(this));
   },
 
@@ -246,19 +246,19 @@ GvaScript.AutoCompleter.prototype = {
 
   _updateChoicesFromAjax: function (val_to_complete, continuation) {
 
-    // copies into local variables, needed for closures below (can't rely on 
+    // copies into local variables, needed for closures below (can't rely on
     // 'this' because 'this' may have changed when the ajax call comes back)
-    var autocompleter = this; 
+    var autocompleter = this;
     var inputElement  = this.inputElement;
 
     inputElement.style.backgroundColor = ""; // remove colorIllegal
 
-    // abort prev ajax request on this input element 
+    // abort prev ajax request on this input element
     if (this._runningAjax[inputElement.name])
       this._runningAjax[inputElement.name].transport.abort();
 
     Element.addClassName(inputElement, this.classes.loading);
-    
+
     var complete_url = this._datasource + val_to_complete;
     this._runningAjax[inputElement.name] = new Ajax.Request(
       complete_url,
@@ -266,29 +266,34 @@ GvaScript.AutoCompleter.prototype = {
        method: this.options.http_method,
        parameters: this.additional_params, // for example {C_ETAT_AVOC : 'AC'}
 
-       // DALNOTE 10.01.09: forcer du JSON dans le body du POST est spcifique 
-       // DMWeb; pour le cas gnral il faut pouvoir envoyer du 
+       // DALNOTE 10.01.09: forcer du JSON dans le body du POST est spcifique
+       // DMWeb; pour le cas gnral il faut pouvoir envoyer du
        // x-www-form-urlencoded ordinaire
        postBody: this.options.http_method == 'post'
-                     ? Object.toJSON(this.additional_params) 
+                     ? Object.toJSON(this.additional_params)
                      : null,
 
        contentType: "text/javascript",
        evalJSON: 'force', // will evaluate even if header != 'application/json'
        onSuccess: function(xhr) {
           // aborted by the onblur handler
-          if(xhr.transport.status == 0) return;
-          
+          if (xhr.transport.status == 0) return;
+
           autocompleter._runningAjax[inputElement.name] = null;
-          if (xhr.responseJSON)
-              continuation(xhr.responseJSON);
+
+          if (xhr.responseJSON) continuation(xhr.responseJSON);
+
+          // autocompleter input already blurred without _blurHandler being
+          // called (autocompleter is strict and needs its choices to
+          // be able to fire its final status
+          if (xhr['blurAfterSuccess']) autocompleter._blurHandler();
        },
        onFailure: function(xhr) {
           autocompleter._runningAjax[inputElement.name] = null;
           autocompleter.displayMessage("pas de rponse du serveur");
        },
        onComplete: function(xhr) {
-          Element.removeClassName(inputElement, 
+          Element.removeClassName(inputElement,
                                   autocompleter.classes.loading);
        }
       });
@@ -306,21 +311,32 @@ GvaScript.AutoCompleter.prototype = {
         Element.addClassName(that.inputElement, that.classes.loading);
         Prototype.getJSON(_url, function(data) {
           var _data_list = data;
-         
+
           if(that._datasource.json_list)
           that._datasource.json_list.split('/').each(function(p) {
             _data_list = _data_list[p];
           });
           Element.removeClassName(that.inputElement, that.classes.loading);
-          
+
           continuation(_data_list);
         });
       }
   },
 
   _updateChoicesFromArray : function(val_to_complete, continuation) {
-    if (this.options.ignorePrefix)
-        continuation(this._datasource);
+    if (this.options.ignorePrefix) {
+      // store the index of the initial value
+      if (val_to_complete) {
+        this._idx_to_hilite = (val_to_complete == ''? 0 : -1);
+        $A(this._datasource).each(function(choice, index) {
+          if(choice.toLowerCase().startsWith(val_to_complete.toLowerCase())) {
+            this._idx_to_hilite = index;
+            throw $break;
+          }
+        }, this);
+      }
+      continuation(this._datasource);
+    }
     else {
       var regex = new RegExp("^" + RegExp.escape(val_to_complete),
                              this.options.caseSensitive ? "" : "i");
@@ -350,15 +366,24 @@ GvaScript.AutoCompleter.prototype = {
 
   // does the reverse of "autocomplete()"
   // doesnot fire if input blurred from click on choice list
-  _blurHandler: function(event) { 
+  _blurHandler: function(event) {
 
     // remove choice list
-    if (this.dropdownDiv)  this._removeDropdownDiv(); 
+    if (this.dropdownDiv)  this._removeDropdownDiv();
 
-    // abort any pending ajax call on this input element
-    if (this._runningAjax[this.inputElement.name]) {
-      this._runningAjax[this.inputElement.name].transport.abort();
-      this._runningAjax[this.inputElement.name] = null;
+    // xhr is still active: waiting for response from server
+    if (_xhr = this._runningAjax[this.inputElement.name]) {
+
+      // if autocompleter is strict, need to wait for xhr to
+      // finish before calling the _blurHandler to fire the
+      // autocompleter's finalState
+      if (this.options.strict) {
+        _xhr['blurAfterSuccess'] = true;
+        return;
+      }
+
+      _xhr.transport.abort();
+      _xhr = null;
       Element.removeClassName(this.inputElement, this.classes.loading);
     }
 
@@ -368,29 +393,29 @@ GvaScript.AutoCompleter.prototype = {
 
       // if value has changed, invalidate previous list of choices
       if (value != this.lastValue) {
-        this.choices = null; 
+        this.choices = null;
       }
 
       // if blank and blankOK, this is a legal value
       if (!value && this.options.blankOK) {
         this._updateDependentFields(this.inputElement, "");
-        this.fireEvent({ type       : "LegalValue", 
-                         value      : "", 
+        this.fireEvent({ type       : "LegalValue",
+                         value      : "",
                          choice     : null,
                          controller : null  }, this.inputElement);
       }
 
       // if choices are known, just inspect status
-      else if (this.choices) {            
+      else if (this.choices) {
         this._fireFinalStatus(this.inputElement, this.choices);
       }
 
       // if not enough chars to get valid choices, this is illegal
-      else if (value.length < this.options.minimumChars) {    
+      else if (value.length < this.options.minimumChars) {
         var return_value = this.fireEvent({
           type: "IllegalValue", value: value
         }, this.inputElement);
-        
+
         if(! return_value) {
           this.inputElement.style.backgroundColor = this.options.colorIllegal;
           this._updateDependentFields(this.inputElement, null);
@@ -399,7 +424,7 @@ GvaScript.AutoCompleter.prototype = {
 
       // otherwise get choices and then inspect status (maybe asynchronously)
       else  {
-        this._updateChoices(this._fireFinalStatus.bind(this, 
+        this._updateChoices(this._fireFinalStatus.bind(this,
                                                        this.inputElement));
       }
     }
@@ -420,7 +445,7 @@ GvaScript.AutoCompleter.prototype = {
     // inspect the choice list to automatically choose the appropriate candidate
     for (var i=0; i < choices.length; i++) {
         var val = this._valueFromChoiceItem(choices[i]);
-        
+
         if (val == input_val) {
             index = i;
             break; // break the loop because this is the best choice
@@ -432,10 +457,10 @@ GvaScript.AutoCompleter.prototype = {
 
     // if automatic choice did not work, but we have only 1 choice, and this is
     // not blank on purpose, then force it into the field
-    if (index === null && choices.length == 1 
-                       && (input_val || !this.options.blankOK )) 
+    if (index === null && choices.length == 1
+                       && (input_val || !this.options.blankOK ))
         index = 0;
-        
+
     if (index !== null) {
         var choice = choices[index];
         var val = this._valueFromChoiceItem(choice);
@@ -444,10 +469,10 @@ GvaScript.AutoCompleter.prototype = {
         this._setValue(val, inputElement);
 
         // for backwards compatibility, we generate a "Complete" event, but
-        // with a fake controller (because the real controller might be in a 
+        // with a fake controller (because the real controller might be in a
         // diffent state).
         this.fireEvent({ type      : "Complete",
-                         referrer  : "blur",    // input blur fired this event 
+                         referrer  : "blur",    // input blur fired this event
                          index     : index,
                          choice    : choice,
                          controller: {choices: choices} }, inputElement);
@@ -456,17 +481,17 @@ GvaScript.AutoCompleter.prototype = {
         this._updateDependentFields(inputElement, choice);
 
         // for new code : generate a "LegalValue" event
-        this.fireEvent({ type       : "LegalValue", 
-                         value      : val, 
+        this.fireEvent({ type       : "LegalValue",
+                         value      : val,
                          choice     : choice,
                          controller : null  }, inputElement);
 
     }
     else {
-        var return_value = this.fireEvent({ 
-          type       : "IllegalValue", 
-          value      : input_val, 
-          controller : null  
+        var return_value = this.fireEvent({
+          type       : "IllegalValue",
+          value      : input_val,
+          controller : null
         }, inputElement);
 
         if(! return_value) {
@@ -477,18 +502,18 @@ GvaScript.AutoCompleter.prototype = {
   },
 
   _updateDependentFields: function(inputElement, choice) {
-        // "choice" might be 
+        // "choice" might be
         //   - an object or nonempty string ==> update dependent fields
         //   - an empty string              ==> clear dependent fields
-        //   - null                         ==> put "ILLEGAL_***" 
+        //   - null                         ==> put "ILLEGAL_***"
         var attr       = inputElement.getAttribute('ac:dependentFields');
-        var dep_fields = attr ? eval("("+attr+")") 
+        var dep_fields = attr ? eval("("+attr+")")
                               : this.options.dependentFields;
         if (!dep_fields) return;
 
         var form       = inputElement.form;
         var name_parts = inputElement.name.split(/\./);
-        
+
         for (var k in dep_fields) {
             name_parts[name_parts.length - 1] = k;
             var related_name    = name_parts.join('.');
@@ -498,7 +523,8 @@ GvaScript.AutoCompleter.prototype = {
                 related_field.value
                     = (value_in_choice == "")        ? ""
                     : (choice === null)              ? "!!ILLEGAL_" + k + "!!"
-                    : (typeof choice == "object")    ? choice[value_in_choice]
+                    : (typeof choice == "object")    ? 
+                      (choice[value_in_choice]       ? choice[value_in_choice] : "")
                     : (typeof choice == "string")    ? choice
                     : "!!UNEXPECTED SOURCE FOR RELATED FIELD!!";
             }
@@ -519,53 +545,53 @@ GvaScript.AutoCompleter.prototype = {
     }
   },
 
-  _ArrowDownHandler: function(event) { 
-    var value = this._getValueToComplete(); 
-    var valueLength = (value || "").length; 
+  _ArrowDownHandler: function(event) {
+    var value = this._getValueToComplete();
+    var valueLength = (value || "").length;
     if (valueLength < this.options.minimumChars)
-      this.displayMessage("liste de choix  partir de " 
+      this.displayMessage("liste de choix  partir de "
                             + this.options.minimumChars + " caractres");
-    else 
+    else
       this._displayChoices();
     Event.stop(event);
   },
 
 
 
-  _keyDownHandler: function(event) { 
+  _keyDownHandler: function(event) {
 
     // invalidate previous lists of choices because value may have changed
-    this.choices = null; 
+    this.choices = null;
     this._removeDropdownDiv();
 
     // cancel pending timeouts because we create a new one
     if (this._timeoutId) clearTimeout(this._timeoutId);
 
-    this._timeLastKeyDown = (new Date()).getTime(); 
-//     if (window.console) console.log('keyDown', this._timeLastKeyDown, event.keyCode); 
-    this._timeoutId = setTimeout(this._checkNewValue.bind(this), 
+    this._timeLastKeyDown = (new Date()).getTime();
+//     if (window.console) console.log('keyDown', this._timeLastKeyDown, event.keyCode);
+    this._timeoutId = setTimeout(this._checkNewValue.bind(this),
                                  this.options.checkNewValDelay);
 
-    // do NOT stop the event here : give back control so that the standard 
-    // browser behaviour can update the value; then come back through a 
+    // do NOT stop the event here : give back control so that the standard
+    // browser behaviour can update the value; then come back through a
     // timeout to update the Autocompleter
   },
 
 
 
-  _checkNewValue: function() { 
-        
+  _checkNewValue: function() {
+
     // abort if the timeout occurs after a blur (no input element)
     if (!this.inputElement) {
 //       if (window.console) console.log('_checkNewValue ... no input elem');
-      return; 
+      return;
     }
 
     // several calls to this function may be queued by setTimeout,
     // so we perform some checks to avoid doing the work twice
     if (this._timeLastCheck > this._timeLastKeyDown) {
 
-//       if (window.console) console.log('_checkNewValue ... done already ', 
+//       if (window.console) console.log('_checkNewValue ... done already ',
 //                   this._timeLastCheck, this._timeLastKeyDown);
 
       return; // the work was done already
@@ -574,7 +600,7 @@ GvaScript.AutoCompleter.prototype = {
     var now = (new Date()).getTime();
 
     var deltaTime = now - this._timeLastKeyDown;
-    if (deltaTime + this.options.deltaTime_tolerance 
+    if (deltaTime + this.options.deltaTime_tolerance
           <  this.options.checkNewValDelay) {
 
 //       if (window.console) console.log('_checkNewValue ... too young ',
@@ -586,8 +612,8 @@ GvaScript.AutoCompleter.prototype = {
 
     this._timeLastCheck = now;
     var value = this._getValueToComplete();
-//     if (window.console) 
-//         console.log('_checkNewValue ... real work [value = %o]  - [lastValue = %o] ', 
+//     if (window.console)
+//         console.log('_checkNewValue ... real work [value = %o]  - [lastValue = %o] ',
 //                              value, this.lastValue);
     this.lastValue = this.lastTypedValue = value;
 
@@ -599,12 +625,12 @@ GvaScript.AutoCompleter.prototype = {
 
           // if, meanwhile, another keyDown occurred, then abort
           if (this._timeLastKeyDown > this._timeLastCheck) {
-//             if (window.console) 
+//             if (window.console)
 //               console.log('after updateChoices .. abort because of keyDown',
 //                           now, this._timeLastKeyDown);
             return;
           }
-          
+
           this.choices = choices;
           if (choices && choices.length > 0) {
             this.inputElement.style.backgroundColor = ""; // remove colorIllegal
@@ -626,8 +652,8 @@ GvaScript.AutoCompleter.prototype = {
   // TODO : for multivalued, should return the value under the cursor,
   // instead returning sytematically the last value
   _getValueToComplete : function(value) {
-     // NOTE: the explicit value as argument is only used from 
-     //_fireFinalStatus(), when we can no longer rely on 
+     // NOTE: the explicit value as argument is only used from
+     //_fireFinalStatus(), when we can no longer rely on
      // this.inputElement.value
     value = value || this.inputElement.value;
     if (this.options.multivalued) {
@@ -637,8 +663,8 @@ GvaScript.AutoCompleter.prototype = {
     return value;
   },
 
-  _setValue : function(value, inputElement) {              
-        // NOTE: the explicit inputElement as argument is only used from 
+  _setValue : function(value, inputElement) {
+        // NOTE: the explicit inputElement as argument is only used from
         // _fireFinalStatus(), when we can no longer rely on this.inputElement
 
     // default inputElement is the one bound to this autocompleter
@@ -649,7 +675,7 @@ GvaScript.AutoCompleter.prototype = {
       var _sep = inputElement.value.match(this.options.multivalue_separator);
       if (_sep) {
         var vals = inputElement.value.split(this.options.multivalue_separator);
-        vals[vals.length-1] = value;      
+        vals[vals.length-1] = value;
         value = vals.join(_sep[0]); // join all vals with first separator found
       }
     }
@@ -662,7 +688,7 @@ GvaScript.AutoCompleter.prototype = {
 
   _typeAhead : function () {
     var curLen     = this.lastTypedValue.length;
-    var index      = this.choiceList.currentHighlightedIndex; 
+    var index      = this.choiceList.currentHighlightedIndex;
     var suggestion = this._valueFromChoice(index);
     var newLen     = suggestion.length;
     this._setValue(suggestion);
@@ -717,11 +743,11 @@ GvaScript.AutoCompleter.prototype = {
     // simulate minWidth on old MSIE (must be AFTER appendChild())
     // maxHeight cannot be simulated untill displayChoices
     if (navigator.userAgent.match(/\bMSIE [456]\b/)) {
-      div.style.width  = this.options.minWidth + "px"; 
+      div.style.width  = this.options.minWidth + "px";
     }
-   
+
     // mouseenter and mouseleave events to control
-    // whether autocompleter has been blurred 
+    // whether autocompleter has been blurred
     var elem = this.inputElement;
     div.observe('mouseenter', function(e) {
       Element.stopObserving(elem, "blur", this.reuse.onblur);
@@ -752,7 +778,7 @@ GvaScript.AutoCompleter.prototype = {
         choiceItemTagName : this.options.choiceItemTagName,
         htmlWrapper       : this.options.htmlWrapper
       });
-
+      cl.currentHighlightedIndex = ac._idx_to_hilite;
 
       // TODO: explain and publish method "choiceElementHTML", or redesign
       // and make it a private method
@@ -761,7 +787,7 @@ GvaScript.AutoCompleter.prototype = {
       }
 
       cl.onHighlight = function(event) {
-        if (ac.options.typeAhead) 
+        if (ac.options.typeAhead)
           ac._typeAhead();
         ac.fireEvent(event, ac.inputElement);
       };
@@ -771,14 +797,14 @@ GvaScript.AutoCompleter.prototype = {
       cl.onCancel = function(event) {
         ac._removeDropdownDiv();
       };
-        
+
       // append div to DOM
       var choices_div = this._mkDropdownDiv();
       // fill div now so that the keymap gets initialized
       cl.fillContainer(choices_div);
       // set height of div for IE6 (no suppport for maxHeight!)
       if (navigator.userAgent.match(/\bMSIE [456]\b/)) {
-        choices_div.style.height = 
+        choices_div.style.height =
           (choices_div.scrollHeight > this.options.maxHeight)?
             this.options.maxHeight + 'px' :
             'auto';
@@ -788,17 +814,17 @@ GvaScript.AutoCompleter.prototype = {
       // the choices list under the input
       // if not, display above.
       // onscreen height needed for displaying the choices list
-      var _h_needed = Element.viewportOffset(this.inputElement)[1] 
-                      + this.inputElement.offsetHeight 
+      var _h_needed = Element.viewportOffset(this.inputElement)[1]
+                      + this.inputElement.offsetHeight
                       + choices_div.offsetHeight;
       var _h_avail  = document.viewport.getHeight();
       // move choices list on top of the input element
-      if(_h_needed >= _h_avail) {  
-        var div_top = choices_div.offsetTop 
-                      - choices_div.offsetHeight 
+      if(_h_needed >= _h_avail) {
+        var div_top = choices_div.offsetTop
+                      - choices_div.offsetHeight
                       - this.inputElement.offsetHeight;
-        if (div_top >= 0)          
-          choices_div.style.top = div_top + 'px'; 
+        if (div_top >= 0)
+          choices_div.style.top = div_top + 'px';
       }
 
       // catch keypress on TAB while choiceList has focus
@@ -810,7 +836,7 @@ GvaScript.AutoCompleter.prototype = {
 
           // generate a "Ping" on the choiceList, like if user had
           // pressed RETURN to select the current highlighted item
-          cl.fireEvent({type : "Ping", 
+          cl.fireEvent({type : "Ping",
                         index: index}, elem, cl.container);
 
           // NO Event.stop() here, because the navigator should
@@ -822,11 +848,11 @@ GvaScript.AutoCompleter.prototype = {
       this.keymap.rules.push(cl.keymap.rules[0]);
 
     }
-    else 
+    else
       this.displayMessage("pas de suggestion");
   },
 
-  _removeDropdownDiv: function() { 
+  _removeDropdownDiv: function() {
     // remove the dropdownDiv that was added previously by _mkDropdownDiv();
     // that div contained either a menu of choices or a message to the user
     if (this.dropdownDiv) {
@@ -838,7 +864,7 @@ GvaScript.AutoCompleter.prototype = {
 
     // if applicable, also remove rules previously pushed by _displayChoices
     if (this.keymap.rules.length > 1)
-      this.keymap.rules.pop(); 
+      this.keymap.rules.pop();
   },
 
   _valueFromChoice: function(index) {
@@ -848,7 +874,7 @@ GvaScript.AutoCompleter.prototype = {
   },
 
   _valueFromChoiceItem: function(choice) {
-    return (typeof choice == "string") ? choice 
+    return (typeof choice == "string") ? choice
                                        : choice[this.options.valueField];
   },
 
@@ -867,7 +893,7 @@ GvaScript.AutoCompleter.prototype = {
       this._removeDropdownDiv();
       if (!this.options.multivalued) {
         this.inputElement.select();
-      } 
+      }
 
       this._updateDependentFields(this.inputElement, this.choices[num]);
 
@@ -875,17 +901,17 @@ GvaScript.AutoCompleter.prototype = {
       // for regular use
       var eventNames =  ["Complete", "LegalValue"];
       // for loop : can't use .each() from prototype.js because it alters "this"
-      for (var i = 0; i < eventNames.length; i++) { 
+      for (var i = 0; i < eventNames.length; i++) {
         this.fireEvent({
           type      : eventNames[i],
-          referrer  : "select",    // choice selection fired this event 
+          referrer  : "select",    // choice selection fired this event
           index     : num,
           choice    : this.choices[num],
-          controller: {choices: this.choices} 
+          controller: {choices: this.choices}
           }, elem, this.inputElement);
       }
     }
   }
 }
 
- 
+
