@@ -27,89 +27,123 @@ GvaScript.Form.Methods = {
 
   to_tree: function(form) {
     form = $(form);
-    
+
     return Hash.expand(GvaScript.Form.to_hash(form));
   },
 
   fill_from_tree : (function() {
-  
+
+    var doc = document; // local variable is faster than global 'document'
+
     // IMPLEMENTATION NOTE : Form.Element.setValue() is quite similar,
     // but our treatment of arrays is different, so we have to reimplement
-    var _fill_from_value = function(elem, val, is_init) {
-      elem = $(elem);
+    var _fill_from_value = function(form, elem, val, is_init) {
 
       // force val into an array
-      if (!(val instanceof Array)) val = [val]; 
+      if (!(val instanceof Array)) val = [val];
+
 
-      // get element type (might be a node list, which we call "collection")
-      var elem_type = elem.type 
-                  || (elem.length !== undefined ? "collection" : "unknown");
       var old_value = null; // needed for value:change custom event
+      var new_value = null;
 
-      switch (elem_type) {
-        case "collection":
-          for (var i=0; i < elem.length; i++) {
-            _fill_from_value(elem.item(i), val, is_init);
-          }
-        break;
-
-        case "checkbox" :
-        case "radio":
-          old_value = elem.getValue();
-          elem.checked = val.include(elem.value);
-        break;
+      switch (elem.type) {
 
         case "text" :
         case "textarea" :
         case "hidden" :
-          old_value = elem.getValue();
-          elem.value = val.join(",");
+          old_value  = elem.value;
+          elem.value = new_value = val.join(",");
+        break;
+
+        case "checkbox" :
+        case "radio":
+          var elem_val  = elem.value;
+          old_value = elem.checked ? elem_val : null;
+
+          // hand-crafted loop through val array (because val.include() is too slow)
+          elem.checked = false;
+          for (var count = val.length; count--;) {
+            if (val[count] == elem_val) {
+                elem.checked = true;
+                break;
+            }
+          }
+          new_value = elem.checked ? elem_val : null;
         break;
 
         case "select-one" :
         case "select-multiple" :
-          old_value = elem.getValue();
-          $A(elem.options).each(function(opt){
-            var opt_value = Form.Element.Serializers.optionValue(opt);
-            opt.selected = val.include(opt_value);
-          });
+          var options = elem.options;
+          var old_values = [],
+              new_values = [];
+          for (var i=0, len=options.length; i<len; i++) {
+            var opt = options[i];
+            var opt_value = opt.value || opt.text;
+            if (opt.selected) old_values.push(opt_value);
+            // hand-crafted loop through val array (because val.include() is too slow
+            opt.selected = false;
+            for (var count = val.length; count--;) {
+              if (val[count] == opt_value) {
+                new_values.push(opt_value);
+                opt.selected = true;
+                break;
+              }
+            }
+          }
+          old_value = old_values.join(",");
+          new_value = new_values.join(",");
         break;
 
         default:
+          // if no element type, might be a node list
+          var elem_length = elem.length;
+          if (elem_length !== undefined) {
+            for (var i=0; i < elem_length; i++) {
+              _fill_from_value(form, elem.item(i), val, is_init);
+            }
+          }
+          else
             throw new Error("unexpected elem type : " + elem.type);
         break;
       } // end switch
 
-      if(elem.fire) // not to be called when elem_type is collection
-      // called as a subsequence of GvaScript.Form.init method
-      if(is_init) 
-      elem.fire('value:init',   {newvalue: elem.getValue()});
-      else                  
-      elem.fire('value:change', {oldvalue: old_value, newvalue: elem.getValue()});
+      // if initializing form
+      //   and form has an init handler registered to its inputs
+      //   and elem has a new_value set
+      // => fire the custom 'value:init' event
+      if (is_init) {
+        if (form.has_init_registered)
+          if (new_value)
+            Element.fire(elem, 'value:init', {newvalue: new_value}); 
+      }
+      else {
+        if (new_value != old_value)
+          Element.fire(elem, 'value:change', {oldvalue: old_value, newvalue: new_value});
+      }
     }
 
     var _fill_from_array = function (form, field_prefix, array, is_init) {
-      for (var i=0; i < array.length; i++) {
+      for (var i=0, len=array.length; i < len; i++) {
         var new_prefix = field_prefix + "." + i;
 
         // if form has a corresponding named element, fill it
         var elem = form[new_prefix];
         if (elem) {
-          _fill_from_value(elem, array[i], is_init);
+          _fill_from_value(form, elem, array[i], is_init);
           continue;
         }
 
         // otherwise try to walk down to a repetition block
 
         // try to find an existing repetition block
-        elem = $(new_prefix);  // TODO : check: is elem in form ?
+        elem = doc.getElementById(new_prefix);  // TODO : check: is elem in form ?
 
         // no repetition block found, try to instanciate one
-        if (!elem) { 
-          var placeholder = $(field_prefix + ".placeholder");
+        if (!elem) {
+          var placeholder = doc.getElementById(field_prefix + ".placeholder");
           if (placeholder && placeholder.repeat) {
             GvaScript.Repeat.add(placeholder, i + 1 - placeholder.repeat.count);
-            elem = $(new_prefix);
+            elem = doc.getElementById(new_prefix);
           }
         }
         // recurse to the repetition block
@@ -119,12 +153,12 @@ GvaScript.Form.Methods = {
         // -> no id's on repeatable blocks are set but need to recurse anyway
 //         if (elem)
         GvaScript.Form.fill_from_tree(form, new_prefix, array[i], is_init);
-      }           
+      }
     }
-           
+
     function fill_from_tree(form, field_prefix, tree, is_init)  {
-      form = $(form);
-     
+      if (form instanceof String) form = $(form);
+
       for (var key in tree) {
         if (!tree.hasOwnProperty(key)) continue;
 
@@ -136,18 +170,18 @@ GvaScript.Form.Methods = {
               val = val ? "true" : "";
               // NO break here
 
-          case "number":
           case "string":
+          case "number":
               var elem = form[new_prefix];
-              if (elem) 
-              _fill_from_value(elem, val, is_init); 
+              if (elem)
+              _fill_from_value(form, elem, val, is_init);
               break;
 
           case "object":
-              if (val instanceof Array) 
-              _fill_from_array(form, new_prefix, val, is_init);
-              else 
-              this.fill_from_tree(form, new_prefix, val, is_init);
+              if (val instanceof Array)
+                _fill_from_array(form, new_prefix, val, is_init);
+              else
+                this.fill_from_tree(form, new_prefix, val, is_init);
               break;
 
           case "function":
@@ -160,25 +194,49 @@ GvaScript.Form.Methods = {
   })(),
 
   autofocus: function(container) {
-    container = $(container);
-    
+
+    if (container instanceof String) 
+      container = document.getElementById(container);
+
+    // replace prototype's down selector
+    // as it performs slowly on IE6
+    var _find_autofocus = function(p_node) {
+      var _kids = p_node.childNodes;
+
+      for(var _idx = 0, len = _kids.length; _idx < len; ) {
+        _kid = _kids[_idx ++];
+
+        if(_kid.nodeType == 1) {
+          if(Element.hasAttribute(_kid, 'autofocus')) {
+            return _kid;
+          }
+          else {
+            var _look_in_descendants = _find_autofocus(_kid);
+            if(_look_in_descendants) return _look_in_descendants;
+          }
+        }
+      }
+    }
+
     if(container) {
-      var target = container.down('[autofocus]');
+      //slow on IE6
+      //var target = container.down('[autofocus]');
+      var target = _find_autofocus(container);
       // TODO : check if target is visible
-      if (target) try {target.activate()} 
+      if (target) try {target.activate()}
                   catch(e){}
     }
   },
 
   /**
     * wrapper around Element.register method.
-    * method wrapped for special handling of form inputs 
+    * method wrapped for special handling of form inputs
     * 'change' and 'init' events
     *
     * all handlers will receive 'event' object as a first argument.
     * 'change' handler will also receive input's oldvalue/newvalue as
     * second and third arguments respectively.
-    * 'init' handler will also receive input's newvalue as a 
+    * 'init' handler will also receive input's newvalue as a
     * second argument.
     *
     * @param {string} query : css selector to match elements
@@ -200,7 +258,7 @@ GvaScript.Form.Methods = {
         case 'change':
           form.register(query, 'focus', function(event) {
               var elt = event._target;
-              elt.store('value', elt.getValue());  
+              elt.store('value', elt.getValue());
           });
 
           form.register(query, 'blur', function(event) {
@@ -210,7 +268,7 @@ GvaScript.Form.Methods = {
 
               if(oldvalue != newvalue) {
                   elt.fire('value:change', {
-                      oldvalue : oldvalue, 
+                      oldvalue : oldvalue,
                       newvalue : newvalue,
                       handler  : handler
                   });
@@ -218,12 +276,17 @@ GvaScript.Form.Methods = {
               }
           });
         break;
-        
+
         // value:init fired by GvaScript.Form.fill_from_tree method
         // used in formElt initialization
         case 'init':
+          // set a flag here in order to fire the 
+          // value:init custom event while initializing
+          // the form 
+          form.has_init_registered = true;
+
           form.register(query, 'value:init', function(event) {
-              handler(event, event.memo.newvalue);        
+              handler(event, event.memo.newvalue);
           });
         break;
 
@@ -235,7 +298,7 @@ GvaScript.Form.Methods = {
 
   /**
     * wrapper around Element.unregister method.
-    * method wrapped for special handling of form inputs 
+    * method wrapped for special handling of form inputs
     * 'change' and 'init' events
     *
     * remove handler attached to eventname for inputs that match query
@@ -245,7 +308,7 @@ GvaScript.Form.Methods = {
     * @param {Funtion} handler : handler to stop firing oneventname
     *                            NOTE: should be identical to what was used in
     *                            register method.
-    *                            {optional} : if not specified, will remove all 
+    *                            {optional} : if not specified, will remove all
     *                            handlers attached to eventname for indicated selector
     * @return undefined
     */
@@ -263,7 +326,7 @@ GvaScript.Form.Methods = {
     }
   }
 }
-  
+
 Object.extend(GvaScript.Form.prototype, function() {
     // private method to initialize and add actions
     // to form's actions bar
@@ -280,7 +343,7 @@ Object.extend(GvaScript.Form.prototype, function() {
         }
     }
 
-    return { 
+    return {
         formElt: null,
         actionsbar: null,
         initialize: function(formElt, options) {
@@ -293,7 +356,9 @@ Object.extend(GvaScript.Form.prototype, function() {
 
                 actionsbar: {},                             // form actions
                 registry: [],                               // list of [elements_selector, event_name, event_handler]
-                
+
+                skipAutofocus : false,
+
                 onInit           : Prototype.emptyFunction,  // called after form initialization
 
                 onRepeatBlockRemove : Prototype.emptyFunction,  // called when a repeatable block gets removed
@@ -318,16 +383,16 @@ Object.extend(GvaScript.Form.prototype, function() {
             $A(this.options.registry).each(function(w) {
                 this.register(w[0], w[1], w[2]);
             }, this);
-            
+
             var that = this;
             // workaround as change event doesnot bubble in IE
             this.formElt.observe('value:change', function(event) {
                 if(event.memo.handler) {
-                    event.memo.handler(event, 
+                    event.memo.handler(event,
                                        event.memo.newvalue,
-                                       event.memo.oldvalue 
+                                       event.memo.oldvalue
                     );
-                    // fire the onChange event passing the event 
+                    // fire the onChange event passing the event
                     // object as an arguement
                     that.fire('Change', event);
                 }
@@ -351,14 +416,15 @@ Object.extend(GvaScript.Form.prototype, function() {
             this.register('input,textarea','change', Prototype.emptyFunction);
 
             // initializing for with data
-            GvaScript.Form.init(this.formElt,    
-                                this.options.datatree, 
-                                this.options.dataprefix);
+            GvaScript.Form.init(this.formElt,
+                                this.options.datatree,
+                                this.options.dataprefix,
+                                this.options.skipAutofocus);
 
             // declaring form as a widget
             this.formElt.store('widget', this);
             this.formElt.addClassName(CSSPREFIX()+'-widget');
-            
+
             // register the instance
             GvaScript.Forms.register(this);
 
@@ -372,7 +438,7 @@ Object.extend(GvaScript.Form.prototype, function() {
         },
 
         // use to submit the for programatically
-        // since the form.submit() doesnot fire the 
+        // since the form.submit() doesnot fire the
         // onsubmit event. doh!
         submitForm: function() {
           // submit method only called if
@@ -384,7 +450,7 @@ Object.extend(GvaScript.Form.prototype, function() {
          * fire the eventName (ex: 'XYZ') on the form instance.
          * basic events supported are: Init, Change, BeforeSubmit, Submit
          *
-         * will first dispatch EarlyResponders defined in GvaScript.Form.EarlyResponders, 
+         * will first dispatch EarlyResponders defined in GvaScript.Form.EarlyResponders,
          * if none returned false, will continue to fire the callback defined on this Form instance.
          * if callback doesnot return false, will continue to dispatch Responders
          * defined in GvaScript.Form.Responders
@@ -397,7 +463,7 @@ Object.extend(GvaScript.Form.prototype, function() {
         fire: function(eventName, arg) {
             var callback_ok = true;
 
-            // -- early responders 
+            // -- early responders
             callback_ok = GvaScript.Form.EarlyResponders.dispatch('on'+eventName, this, arg);
             if(callback_ok === false) return false;
 
@@ -408,7 +474,7 @@ Object.extend(GvaScript.Form.prototype, function() {
             }
 
             // -- late responders
-            callback_ok = GvaScript.Form.Responders.dispatch('on'+eventName, this, arg) 
+            callback_ok = GvaScript.Form.Responders.dispatch('on'+eventName, this, arg)
 
             return (callback_ok !== false);
         },
@@ -425,17 +491,17 @@ Object.extend(GvaScript.Form.prototype, function() {
             }
         }
     }
-}());        
+}());
 
 /**
- * GvaScript.Forms : 
- * - holds references to all GvaScript.Form instances indentified 
+ * GvaScript.Forms :
+ * - holds references to all GvaScript.Form instances indentified
  *   by the instance.getId() method.
  *   handy to get GvaScript.Form instance based on the form id.
  *
- * - holds general observers to be executed on all GvaScript.Form 
+ * - holds general observers to be executed on all GvaScript.Form
  *   instances
- */ 
+ */
 
 GvaScript.Form.EarlyResponders = {
   responders: [],
@@ -456,7 +522,7 @@ GvaScript.Form.EarlyResponders = {
   dispatch: function(eventName, form, arg) {
       var falsy_observer = this.any(function(responder) {
           if(Object.isFunction(responder[eventName])) {
-              return (responder[eventName](form, arg) === false ? true : false); 
+              return (responder[eventName](form, arg) === false ? true : false);
           }
       });
       return !falsy_observer;
@@ -483,7 +549,7 @@ GvaScript.Form.Responders = {
   dispatch: function(eventName, form, arg) {
       var falsy_observer = this.any(function(responder) {
           if(Object.isFunction(responder[eventName])) {
-              return (responder[eventName](form, arg) === false ? true : false); 
+              return (responder[eventName](form, arg) === false ? true : false);
           }
       });
       return !falsy_observer;
@@ -493,7 +559,7 @@ Object.extend(GvaScript.Form.Responders, Enumerable);
 
 GvaScript.Forms = {
     forms: $A(),
-    
+
     register: function(form) {
       this.unregister(form);
       this.forms.push(form);
@@ -504,7 +570,7 @@ GvaScript.Forms = {
       if(!form) return;
 
       if(typeof form == 'string') form = this.get(form);
-      
+
       // nothing to unregister
       if(!form) return false;
 
@@ -516,33 +582,34 @@ GvaScript.Forms = {
 
     get: function(id) {
       return this.forms.find(function(f) {return f.getId() == id});
-    } 
+    }
 }
 
 // GvaScript.Form helpers and methods
 Object.extend(GvaScript.Form, GvaScript.Form.Methods);
 Object.extend(GvaScript.Form, {
-  init: function(form, tree, field_prefix) {
+  init: function(form, tree, field_prefix, skipAutofocus) {
     form = $(form);
 
     GvaScript.Repeat.init(form);
-    GvaScript.Form.fill_from_tree(form, 
-                                  field_prefix || "", 
+    GvaScript.Form.fill_from_tree(form,
+                                  field_prefix || "",
                                   tree || {},
                                   true);
 
-    GvaScript.Form.autofocus(form);
+    if (!skipAutofocus)
+      GvaScript.Form.autofocus(form);
   },
 
   add: function(repeat_name, count) {
     var n_blocks = GvaScript.Repeat.add(repeat_name, count);
     var last_block = repeat_name + "." + (n_blocks - 1);
     GvaScript.Form.autofocus(last_block);
-    
+
     // get form owner of block
     if(_block = $(last_block)) {
       _form = _block.up('form');
-      // check if form has a GvaSCript.Form instance 
+      // check if form has a GvaSCript.Form instance
       // wrapped around it
       if(_form) {
         if(_gva_form = GvaScript.Forms.get(_form.identify())) {
@@ -575,11 +642,11 @@ Object.extend(GvaScript.Form, {
       tree  = GvaScript.Form.to_tree(form);
 
       var parts = repeat_name.split(/\./);
-      for (var i = 0 ; i < parts.length; i++) {
+      for (var i = 0, len=parts.length ; i < len; i++) {
         if (!tree) break;
         tree = tree[parts[i]];
       }
-      
+
       // remove rows below, and shift rows above
       if (tree && tree instanceof Array) {
         tree.splice(remove_ix, 1);
@@ -590,7 +657,7 @@ Object.extend(GvaScript.Form, {
     }
 
     // call Repeat.remove() to remove from DOM
-    // and if live_update, to remove and reproduce 
+    // and if live_update, to remove and reproduce
     // the blocks below with correct renumerations
     GvaScript.Repeat.remove(repetition_block, live_update);
 
@@ -601,7 +668,7 @@ Object.extend(GvaScript.Form, {
       GvaScript.Form.fill_from_tree(form, repeat_name, tree);
     }
 
-    // check if form has a GvaSCript.Form instance 
+    // check if form has a GvaSCript.Form instance
     // wrapped around it
     if(_gva_form = GvaScript.Forms.get(form.identify())) {
       _gva_form.fire('RepeatBlockRemove', [repeat_name.split('.').last(), repeat_name + '.' + remove_ix]);
